@@ -15,10 +15,10 @@ class AndroidYowsupEnv(YowsupEnv):
                 "YHNtYoIvt5R3X6YZylbPftF/8ayWTALBgcqhkjOOAQDBQADLwAwLAIUAKYCp0d6z4QQdyN74JDfQ2WCyi8CFDUM4CaNB+ceVXd" \
                 "KtOrNTQcc0e+t"
 
-    _MD5_CLASSES = "zkrJMO7CuBg3gV4h3ZPrbg=="
+    _MD5_CLASSES = "7Ai8bMBxBeO5SFSLutDs8A=="
     _KEY = "/UIGKU1FVQa+ATM2A0za7G2KI9S/CwPYjgAbc67v7ep42eO/WeTLx1lb1cHwxpsEgF4+PmYpLd2YpGUdX/A2JQitsHzDwgcdBpUf7psX1BU="
 
-    _VERSION = "2.11.471"
+    _VERSION = "2.12.96"
     _OS_NAME= "Android"
     _OS_VERSION = "4.3"
     _DEVICE_NAME = "GalaxyS3"
@@ -52,7 +52,11 @@ class AndroidYowsupEnv(YowsupEnv):
             ipad.append(0x36 ^ keyDecoded[i])
         hash = hashlib.sha1()
         subHash = hashlib.sha1()
-        subHash.update(ipad + data)
-        hash.update(opad + subHash.digest())
+        try:
+            subHash.update(ipad + data)
+            hash.update(opad + subHash.digest())
+        except TypeError:
+            subHash.update(bytes(ipad + data))
+            hash.update(bytes(opad + subHash.digest()))
         result = base64.b64encode(hash.digest())
-        return result
\ No newline at end of file
+        return result
