@@ -18,7 +18,8 @@ class YowAuthenticationProtocolLayer(YowProtocolLayer):
             "stream:features": (self.handleStreamFeatures, None),
             "failure": (self.handleFailure, None),
             "success": (self.handleSuccess, None),
-            "challenge": (self.handleChallenge, None)
+            "challenge": (self.handleChallenge, None),
+            "stream:error": (self.handleStreamError, None),
         }
         super(YowAuthenticationProtocolLayer, self).__init__(handleMap)
         self.credentials = None
@@ -50,6 +51,7 @@ class YowAuthenticationProtocolLayer(YowProtocolLayer):
     ###recieved node handlers handlers
     def handleStreamFeatures(self, node):
         nodeEntity = StreamFeaturesProtocolEntity.fromProtocolTreeNode(node)
+        self.toUpper(nodeEntity)
 
     def handleSuccess(self, node):
         if(node.data != None): StorageTools.writeNonce(self.credentials[0],node.data)
@@ -68,6 +70,15 @@ class YowAuthenticationProtocolLayer(YowProtocolLayer):
         nodeEntity = ChallengeProtocolEntity.fromProtocolTreeNode(node)
         self._sendResponse(nodeEntity.getNonce())
 
+    def handleStreamError(self, node):
+        if node.getChild("text"):
+            nodeEntity = StreamErrorConflictProtocolEntity.fromProtocolTreeNode(node)
+        elif node.getChild("ack"):
+            nodeEntity = StreamErrorAckProtocolEntity.fromProtocolTreeNode(node)
+        else:
+            raise AuthError("Unhandled stream:error node:\n%s" % node)
+        self.toUpper(nodeEntity)
+
     ##senders
     def _sendFeatures(self):
         self.entityToLower(StreamFeaturesProtocolEntity(["readreceipts", "groups_v2", "privacy", "presence"]))
@@ -124,4 +135,4 @@ class YowAuthenticationProtocolLayer(YowProtocolLayer):
         encoded = outputKey.encodeMessage(nums, 0, 4, len(nums) - 4)
         authBlob = "".join(map(chr, encoded))
 
-        return (inputKey, outputKey, authBlob)
\ No newline at end of file
+        return (inputKey, outputKey, authBlob)
