@@ -20,6 +20,8 @@ class YowContactsIqProtocolLayer(YowProtocolLayer):
                 self.toUpper(AddContactNotificationProtocolEntity.fromProtocolTreeNode(node))
             elif node.getChild("update"):
                 self.toUpper(UpdateContactNotificationProtocolEntity.fromProtocolTreeNode(node))
+            elif node.getChild("sync"):
+                self.toUpper(ContactsSyncNotificationProtocolEntity.fromProtocolTreeNode(node))
             else:
                 self.raiseErrorForNode(node)
 
