@@ -12,6 +12,7 @@ class GroupsNotificationProtocolEntity(NotificationProtocolEntity):
     def __init__(self, _id,  _from, timestamp, notify, participant, offline):
         super(GroupsNotificationProtocolEntity, self).__init__("w:gp2", _id, _from, timestamp, notify, offline)
         self.setParticipant(participant)
+        self.setGroupId(_from)
 
     def setParticipant(self, participant):
         self._participant = participant
@@ -19,6 +20,13 @@ class GroupsNotificationProtocolEntity(NotificationProtocolEntity):
     def getParticipant(self, full = True):
         return self._participant if full else self._participant.split('@')[0]
 
+    def getGroupId(self):
+        return self._id
+
+    def setGroupId(self, groupId):
+        self._id = groupId
+
+
     def __str__(self):
         out = super(GroupsNotificationProtocolEntity, self).__str__()
         out += "Participant: %s\n" % self.getParticipant()
@@ -31,7 +39,8 @@ class GroupsNotificationProtocolEntity(NotificationProtocolEntity):
 
     @staticmethod
     def fromProtocolTreeNode(node):
-        entity = NotificationProtocolEntity.fromProtocolTreeNode(node)
+        entity = super(GroupsNotificationProtocolEntity, GroupsNotificationProtocolEntity).fromProtocolTreeNode(node)
         entity.__class__ = GroupsNotificationProtocolEntity
         entity.setParticipant(node.getAttributeValue("participant"))
-        return entity
\ No newline at end of file
+        entity.setGroupId(node.getAttributeValue("from"))
+        return entity
