@@ -1,3 +1,5 @@
+import time
+
 from yowsup.structs import ProtocolEntity, ProtocolTreeNode
 from .receipt import ReceiptProtocolEntity
 class OutgoingReceiptProtocolEntity(ReceiptProtocolEntity):
@@ -15,20 +17,28 @@ class OutgoingReceiptProtocolEntity(ReceiptProtocolEntity):
     <receipt offline="0" from="4915225256022@s.whatsapp.net" id="1415577964-1" t="1415578027"></receipt>
     '''
 
-    def __init__(self, _id, to, read = False):
+    def __init__(self, _id, to, read = False, participant = None, callId = None):
         super(OutgoingReceiptProtocolEntity, self).__init__(_id)
-        self.setOutgoingData(to, read)
+        self.setOutgoingData(to, read, participant, callId)
 
-    def setOutgoingData(self, to, read):
+    def setOutgoingData(self, to, read, participant, callId):
         self.to = to
         self.read = read
+        self.participant = participant
+        self.callId = callId
     
     def toProtocolTreeNode(self):
         node = super(OutgoingReceiptProtocolEntity, self).toProtocolTreeNode()
         if self.read:
             node.setAttribute("type", "read")
+        if self.participant:
+            node.setAttribute("participant", self.participant)
+        if self.callId:
+            offer = ProtocolTreeNode("offer", {"call-id": self.callId})
+            node.addChild(offer)
 
         node.setAttribute("to", self.to)
+        node.setAttribute("t", str(int(time.time())))
 
         return node
 
@@ -44,5 +54,6 @@ class OutgoingReceiptProtocolEntity(ReceiptProtocolEntity):
         return OutgoingReceiptProtocolEntity(
             node.getAttributeValue("id"),
             node.getAttributeValue("to"),
-            node.getAttributeValue("type")
+            node.getAttributeValue("type"),
+            node.getAttributeValue("participant")
             )
